"""Jax model export utilities."""

import inspect
import logging

import tensorflow as tf
import tf2onnx
from equinox.internal._finalise_jaxpr import finalise_fn
from jax._src.stages import Wrapped
from jax.experimental import jax2tf
from onnx.onnx_pb import ModelProto

from kinfer.rust_bindings import PyInputType, PyModelMetadata

logger = logging.getLogger(__name__)


def export_fn(
    model: Wrapped,
    metadata: PyModelMetadata,
    *,
    opset: int = 13,
) -> ModelProto:
    """Export a JAX function to ONNX.

    Args:
        model: The model to export.
        metadata: The metadata for the model.
        opset: The ONNX opset to use.

    Returns:
        The ONNX model as a `ModelProto`.
    """
    if not isinstance(model, Wrapped):
        raise ValueError("Model must be a Wrapped function")

    params = inspect.signature(model).parameters
    input_names = list(params.keys())

    # Gets the dummy input tensors for exporting the model.
    tf_args = []
    for name in input_names:
        shape = PyInputType(name).get_shape(metadata)
        tf_args.append(tf.TensorSpec(shape, tf.float32, name=name))

    finalised_fn = finalise_fn(model)
    tf_fn = tf.function(jax2tf.convert(finalised_fn, enable_xla=False))

    model_proto, _ = tf2onnx.convert.from_function(
        tf_fn,
        input_signature=tf_args,
        opset=opset,
        large_model=False,
    )
    return model_proto



"""Plot NDJSON logs saved by kinfer."""

import argparse
import json
import logging
from pathlib import Path
from typing import Optional, Union

import matplotlib.pyplot as plt
import numpy as np

# Set up logger
logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s")


def read_ndjson(filepath: str) -> list[dict]:
    """Read NDJSON file and return list of parsed objects."""
    data = []
    with open(filepath, "r") as f:
        for line in f:
            line = line.strip()
            if line:
                data.append(json.loads(line))
    return data


def skip_initial_data(data: list[dict], skip_seconds: float) -> list[dict]:
    """Skip the first n seconds of data based on timestamps."""
    if not data or skip_seconds <= 0.0:
        return data

    # Extract timestamps and convert to seconds relative to first timestamp
    timestamps = [d["t_us"] for d in data]
    t_start = timestamps[0]
    times = [(t - t_start) / 1e6 for t in timestamps]  # Convert to seconds

    # Find indices where time >= skip_seconds
    skip_indices = [i for i, t in enumerate(times) if t >= skip_seconds]
    if not skip_indices:
        logger.info("All data points are within the skip period (%.2f seconds). No data to plot.", skip_seconds)
        return []

    # Filter data
    start_idx = skip_indices[0]
    filtered_data = data[start_idx:]
    logger.info("Skipped first %.2f seconds (%d data points)", skip_seconds, start_idx)

    return filtered_data


def plot_data(data: list[dict], save_path: Optional[Union[str, Path]] = None) -> None:
    """Plot all data fields from the NDJSON."""
    if not data:
        logger.info("No data to plot")
        return

    # Extract timestamps and convert to seconds relative to first timestamp
    timestamps = [d["t_us"] for d in data]
    t_start = timestamps[0]
    times = [(t - t_start) / 1e6 for t in timestamps]  # Convert to seconds

    # Extract data arrays
    joint_angles = np.array([d["joint_angles"] for d in data if d["joint_angles"] is not None])
    joint_vels = np.array([d["joint_vels"] for d in data if d["joint_vels"] is not None])
    projected_g = np.array([d["projected_g"] for d in data if d["projected_g"] is not None])
    accel = np.array([d["accel"] for d in data if d["accel"] is not None])
    command = np.array([d["command"] for d in data if d["command"] is not None])
    output = np.array([d["output"] for d in data if d["output"] is not None])

    # Create subplots
    fig, axes = plt.subplots(3, 2, figsize=(15, 12))
    fig.suptitle("Robot Data Over Time", fontsize=16)

    if len(joint_angles) > 0:
        ax = axes[0, 0]
        for i in range(joint_angles.shape[1]):
            ax.plot(times[: len(joint_angles)], joint_angles[:, i], alpha=0.7, linewidth=0.8)
        ax.set_title("Joint Angles")
        ax.set_xlabel("Time (s)")
        ax.set_ylabel("Angle (rad)")
        ax.grid(True, alpha=0.3)

    if len(joint_vels) > 0:
        ax = axes[0, 1]
        for i in range(joint_vels.shape[1]):
            ax.plot(times[: len(joint_vels)], joint_vels[:, i], alpha=0.7, linewidth=0.8)
        ax.set_title("Joint Velocities")
        ax.set_xlabel("Time (s)")
        ax.set_ylabel("Velocity (rad/s)")
        ax.grid(True, alpha=0.3)

    if len(projected_g) > 0:
        ax = axes[1, 0]
        labels = ["X", "Y", "Z"]
        for i in range(projected_g.shape[1]):
            ax.plot(times[: len(projected_g)], projected_g[:, i], label=labels[i], linewidth=1.5)
        ax.set_title("Projected Gravity")
        ax.set_xlabel("Time (s)")
        ax.set_ylabel("Acceleration (m/s²)")
        ax.legend()
        ax.grid(True, alpha=0.3)

    if len(accel) > 0:
        ax = axes[1, 1]
        labels = ["X", "Y", "Z"]
        for i in range(accel.shape[1]):
            ax.plot(times[: len(accel)], accel[:, i], label=labels[i], linewidth=1.5)
        ax.set_title("Acceleration")
        ax.set_xlabel("Time (s)")
        ax.set_ylabel("Acceleration (m/s²)")
        ax.legend()
        ax.grid(True, alpha=0.3)

    if len(command) > 0:
        ax = axes[2, 0]
        for i in range(command.shape[1]):
            ax.plot(times[: len(command)], command[:, i], label=f"Cmd {i}", linewidth=1.2)
        ax.set_title("Command")
        ax.set_xlabel("Time (s)")
        ax.set_ylabel("Command Value")
        ax.legend()
        ax.grid(True, alpha=0.3)

    if len(output) > 0:
        ax = axes[2, 1]
        for i in range(output.shape[1]):
            ax.plot(times[: len(output)], output[:, i], alpha=0.7, linewidth=0.8)
        ax.set_title("Output")
        ax.set_xlabel("Time (s)")
        ax.set_ylabel("Output Value")
        ax.grid(True, alpha=0.3)

    plt.tight_layout()

    if save_path:
        plt.savefig(save_path, dpi=300, bbox_inches="tight")
        logger.info("Plot saved to: %s", save_path)
        plt.close()
    else:
        plt.show()


def main() -> None:
    parser = argparse.ArgumentParser(description="Plot NDJSON logs saved by kinfer")
    parser.add_argument("filepath", help="Path to the NDJSON file to plot")
    parser.add_argument("--skip", type=float, default=0.0, help="Skip the first n seconds of data")
    parser.add_argument("--save", action="store_true", help="Save the plot to a PNG file in a plots folder")
    args = parser.parse_args()

    filepath = args.filepath
    if not Path(filepath).exists():
        logger.info("File not found: %s", filepath)
        return

    logger.info("Reading data from %s...", filepath)
    data = read_ndjson(filepath)
    logger.info("Loaded %d data points", len(data))

    filtered_data = skip_initial_data(data, args.skip)

    save_path = None
    if args.save:
        # Create save path in plots folder with same name but .png extension
        input_path = Path(filepath)
        plots_dir = input_path.parent / "plots"
        plots_dir.mkdir(exist_ok=True)

        # Change extension from .ndjson to .png
        filename = input_path.stem + ".png"
        save_path = str(plots_dir / filename)

    plot_data(filtered_data, save_path)


if __name__ == "__main__":
    main()



"""Runs common tests on K-Infer components."""

import json

from kinfer.rust_bindings import PyInputType, PyModelMetadata, metadata_from_json


def test_model_metadata_serialization() -> None:
    metadata = PyModelMetadata(
        joint_names=["joint1", "joint2"],
        num_commands=2,
        carry_size=[1],
    )

    # Checks serialization and deserialization.
    metadata_json = metadata.to_json()
    assert metadata_from_json(metadata_json) == metadata

    # Checks equality.
    metadata_copy = json.loads(metadata_json)
    metadata_copy["carry_size"] = [2]
    assert metadata_from_json(json.dumps(metadata_copy)) != metadata


def test_input_type_serialization() -> None:
    metadata = PyModelMetadata(
        joint_names=["joint1", "joint2"],
        num_commands=2,
        carry_size=[1],
    )

    # Tests joint angles.
    input_type = PyInputType(input_type="joint_angles")
    assert input_type.get_shape(metadata) == [2]

    # Tests joint angular velocities.
    input_type = PyInputType(input_type="joint_angular_velocities")
    assert input_type.get_shape(metadata) == [2]

    # Tests projected gravity.
    input_type = PyInputType(input_type="projected_gravity")
    assert input_type.get_shape(metadata) == [3]

    # Tests accelerometer.
    input_type = PyInputType(input_type="accelerometer")
    assert input_type.get_shape(metadata) == [3]

    # Tests gyroscope.
    input_type = PyInputType(input_type="gyroscope")
    assert input_type.get_shape(metadata) == [3]

    # Tests command.
    input_type = PyInputType(input_type="command")
    assert input_type.get_shape(metadata) == [2]

    # Tests time.
    input_type = PyInputType(input_type="time")
    assert input_type.get_shape(metadata) == [1]



"""Tests for model inference functionality on a JAX model."""

import logging
import tempfile
from pathlib import Path
from typing import Sequence

import jax
import numpy as np
from jax import numpy as jnp

from kinfer.export.jax import export_fn
from kinfer.export.serialize import pack
from kinfer.rust_bindings import ModelProviderABC, PyModelMetadata, PyModelRunner

logger = logging.getLogger(__name__)

JOINT_NAMES = ["left_arm", "right_arm", "left_leg", "right_leg"]
NUM_JOINTS = len(JOINT_NAMES)
CARRY_SIZE = 10
NUM_COMMANDS = 4


@jax.jit
def init_fn() -> jnp.ndarray:
    return jnp.zeros((CARRY_SIZE,))


@jax.jit
def step_fn(
    joint_angles: jnp.ndarray,
    joint_angular_velocities: jnp.ndarray,
    projected_gravity: jnp.ndarray,
    accelerometer: jnp.ndarray,
    gyroscope: jnp.ndarray,
    command: jnp.ndarray,
    time: jnp.ndarray,
    carry: jnp.ndarray,
) -> tuple[jnp.ndarray, jnp.ndarray]:
    output = (
        joint_angles.mean()
        + joint_angular_velocities.mean()
        + projected_gravity.mean()
        + accelerometer.mean()
        + gyroscope.mean()
        + command.mean()
        + jnp.cos(time).mean()
        + jnp.sin(time).mean()
        + carry.mean()
    ) * joint_angles
    next_carry = carry + 1
    return output, next_carry


def test_export(tmpdir: Path) -> None:
    metadata = PyModelMetadata(
        joint_names=JOINT_NAMES,
        num_commands=NUM_COMMANDS,
        carry_size=[CARRY_SIZE],
    )

    init_fn_onnx = export_fn(init_fn, metadata)
    step_fn_onnx = export_fn(step_fn, metadata)
    kinfer_model = pack(init_fn_onnx, step_fn_onnx, metadata)

    # Saves the model to disk.
    root_dir = Path(tmpdir)
    (kinfer_path := root_dir / "model.kinfer").write_bytes(kinfer_model)

    num_actions = 0

    class DummyModelProvider(ModelProviderABC):
        def get_inputs(self, input_types: Sequence[str], metadata: PyModelMetadata) -> dict[str, np.ndarray]:
            return_values: dict[str, np.ndarray] = {}
            for input_type in input_types:
                match input_type:
                    case "joint_angles":
                        return_values["joint_angles"] = np.random.randn(NUM_JOINTS)
                    case "joint_angular_velocities":
                        return_values["joint_angular_velocities"] = np.random.randn(NUM_JOINTS)
                    case "projected_gravity":
                        return_values["projected_gravity"] = np.random.randn(3)
                    case "accelerometer":
                        return_values["accelerometer"] = np.random.randn(3)
                    case "gyroscope":
                        return_values["gyroscope"] = np.random.randn(3)
                    case "command":
                        return_values["command"] = np.random.randn(NUM_COMMANDS)
                    case "time":
                        return_values["time"] = np.random.randn(1)
                    case _:
                        raise ValueError(f"Unknown input type: {input_type}")
            return return_values

        def take_action(self, action: np.ndarray, metadata: PyModelMetadata) -> None:
            assert metadata.joint_names == JOINT_NAMES  # type: ignore[attr-defined]
            assert action.shape == (NUM_JOINTS,)
            nonlocal num_actions
            num_actions += 1

    # Creates a model runner from the kinfer model.
    model_provider = DummyModelProvider()
    model_runner = PyModelRunner(str(kinfer_path), model_provider)

    carry = model_runner.init()
    assert carry.shape == (CARRY_SIZE,)
    for _ in range(3):
        output, carry = model_runner.step(carry)
        model_runner.take_action(output)
        assert carry.shape == (CARRY_SIZE,), f"Carry shape: {carry.shape}"

    assert num_actions == 3


if __name__ == "__main__":
    # python -m tests.test_jax
    with tempfile.TemporaryDirectory() as tmpdir:
        test_export(Path(tmpdir))



"""PyTorch model export utilities."""

__all__ = [
    "export_fn",
]

import io
from typing import cast

import onnx
import torch
from onnx.onnx_pb import ModelProto
from torch._C import FunctionSchema

from kinfer.rust_bindings import PyInputType, PyModelMetadata


def export_fn(
    model: torch.jit.ScriptFunction,
    metadata: PyModelMetadata,
) -> ModelProto:
    """Exports a PyTorch function to ONNX.

    Args:
        model: The model to export.
        metadata: The metadata for the model.

    Returns:
        The ONNX model as a `ModelProto`.
    """
    if not isinstance(model, torch.jit.ScriptFunction):
        raise ValueError("Model must be a torch.jit.ScriptFunction")

    schema = cast(FunctionSchema, model.schema)
    input_names = [arg.name for arg in schema.arguments]

    # Gets the dummy input tensors for exporting the model.
    args = []
    for name in input_names:
        shape = PyInputType(name).get_shape(metadata)
        args.append(torch.zeros(shape))

    buffer = io.BytesIO()
    torch.onnx.export(
        model=model,
        f=buffer,  # type: ignore[arg-type]
        args=tuple(args),
        input_names=input_names,
        external_data=False,
    )
    buffer.seek(0)
    model_bytes = buffer.read()
    return onnx.load_from_string(model_bytes)



"""Tests for model inference functionality on a PyTorch model."""

import logging
import tarfile
import tempfile
import time
from pathlib import Path
from typing import Sequence

import numpy as np
import onnxruntime
import torch
from torch import Tensor

from kinfer.export.pytorch import export_fn
from kinfer.export.serialize import pack
from kinfer.rust_bindings import ModelProviderABC, PyModelMetadata, PyModelRunner, PyModelRuntime, metadata_from_json

logger = logging.getLogger(__name__)

JOINT_NAMES = ["left_arm", "right_arm", "left_leg", "right_leg"]
NUM_JOINTS = len(JOINT_NAMES)
CARRY_SIZE = 10
NUM_COMMANDS = 4


@torch.jit.script
def init_fn() -> Tensor:
    return torch.zeros((10,))


@torch.jit.script
def step_fn(
    joint_angles: Tensor,
    joint_angular_velocities: Tensor,
    projected_gravity: Tensor,
    accelerometer: Tensor,
    gyroscope: Tensor,
    command: Tensor,
    time: Tensor,
    carry: Tensor,
) -> tuple[Tensor, Tensor]:
    output = (
        joint_angles.mean()
        + joint_angular_velocities.mean()
        + projected_gravity.mean()
        + accelerometer.mean()
        + gyroscope.mean()
        + command.mean()
        + torch.cos(time).mean()
        + torch.sin(time).mean()
        + carry.mean()
    ) * joint_angles
    next_carry = carry + 1
    return output, next_carry


def test_export(tmpdir: Path) -> None:
    metadata = PyModelMetadata(
        joint_names=JOINT_NAMES,
        num_commands=NUM_COMMANDS,
        carry_size=[CARRY_SIZE],
    )

    init_fn_onnx = export_fn(init_fn, metadata)
    step_fn_onnx = export_fn(step_fn, metadata)
    kinfer_model = pack(init_fn_onnx, step_fn_onnx, metadata)

    # Saves the model to disk.
    root_dir = Path(tmpdir)
    (kinfer_path := root_dir / "model.kinfer").write_bytes(kinfer_model)

    # Ensures that we can open the file like a regular tar file.
    with tarfile.open(kinfer_path, "r:gz") as tar:
        assert tar.getnames() == ["init_fn.onnx", "step_fn.onnx", "metadata.json"]

        # Checks that joint_names.json is valid JSON.
        if (fpath := tar.extractfile("metadata.json")) is None:
            raise ValueError("metadata.json not found")
        metadata = metadata_from_json(fpath.read().decode("utf-8"))
        assert metadata.joint_names == JOINT_NAMES  # type: ignore[attr-defined]

        # Validates that we can construct a session in Python.
        if (fpath := tar.extractfile("init_fn.onnx")) is None:
            raise ValueError("init_fn.onnx not found")
        init_session = onnxruntime.InferenceSession(fpath.read())
        assert init_session.get_modelmeta().graph_name == "main_graph"
        if (fpath := tar.extractfile("step_fn.onnx")) is None:
            raise ValueError("step_fn.onnx not found")
        step_session = onnxruntime.InferenceSession(fpath.read())
        assert step_session.get_modelmeta().graph_name == "main_graph"

    num_actions = 0

    class DummyModelProvider(ModelProviderABC):
        def get_inputs(self, input_types: Sequence[str], metadata: PyModelMetadata) -> dict[str, np.ndarray]:
            return_values: dict[str, np.ndarray] = {}
            for input_type in input_types:
                match input_type:
                    case "joint_angles":
                        return_values["joint_angles"] = np.random.randn(NUM_JOINTS)
                    case "joint_angular_velocities":
                        return_values["joint_angular_velocities"] = np.random.randn(NUM_JOINTS)
                    case "projected_gravity":
                        return_values["projected_gravity"] = np.random.randn(3)
                    case "accelerometer":
                        return_values["accelerometer"] = np.random.randn(3)
                    case "gyroscope":
                        return_values["gyroscope"] = np.random.randn(3)
                    case "command":
                        return_values["command"] = np.random.randn(NUM_COMMANDS)
                    case "time":
                        return_values["time"] = np.random.randn(1)
                    case _:
                        raise ValueError(f"Unknown input type: {input_type}")
            return return_values

        def take_action(self, action: np.ndarray, metadata: PyModelMetadata) -> None:
            assert metadata.joint_names == JOINT_NAMES  # type: ignore[attr-defined]
            assert action.shape == (NUM_JOINTS,)
            nonlocal num_actions
            num_actions += 1

    # Creates a model runner from the kinfer model.
    model_provider = DummyModelProvider()
    model_runner = PyModelRunner(str(kinfer_path), model_provider)

    carry = model_runner.init()
    assert carry.shape == (CARRY_SIZE,)
    for _ in range(3):
        output, carry = model_runner.step(carry)
        model_runner.take_action(output)
        assert carry.shape == (CARRY_SIZE,), f"Carry shape: {carry.shape}"
    assert num_actions == 3

    # Tests the runtime, which runs in a separate Rust thread.
    dt = 10
    model_runtime = PyModelRuntime(model_runner, dt)
    model_runtime.start()
    time.sleep(dt * 4.5 / 1000)
    model_runtime.stop()
    assert num_actions == 8, f"num_actions: {num_actions}"


if __name__ == "__main__":
    # python -m tests.test_pytorch
    with tempfile.TemporaryDirectory() as tmpdir:
        test_export(Path(tmpdir))



"""Functions for serializing and deserializing models."""

__all__ = [
    "pack",
]

import io
import logging
import tarfile

from onnx.onnx_pb import ModelProto

from kinfer.rust_bindings import PyInputType, PyModelMetadata

logger = logging.getLogger(__name__)


def pack(
    init_fn: ModelProto,
    step_fn: ModelProto,
    metadata: PyModelMetadata,
) -> bytes:
    """Packs the initialization function and step function into a directory.

    Args:
        init_fn: The initialization function.
        step_fn: The step function.
        metadata: The metadata for the model.
    """
    num_joints = len(metadata.joint_names)  # type: ignore[attr-defined]

    # Checks the `init` function.
    if len(init_fn.graph.input) > 0:
        raise ValueError(f"`init` function should not have any inputs! Got {len(init_fn.graph.input)}")
    if len(init_fn.graph.output) != 1:
        raise ValueError(f"`init` function should have exactly 1 output! Got {len(init_fn.graph.output)}")
    init_carry = init_fn.graph.output[0]
    init_carry_shape = tuple(dim.dim_value for dim in init_carry.type.tensor_type.shape.dim)

    if metadata.carry_size != init_carry_shape:  # type: ignore[attr-defined]
        logger.warning(
            "Updating carry size from %s to %s to match the `init` function",
            metadata.carry_size,  # type: ignore[attr-defined]
            init_carry_shape,
        )
        metadata.carry_size = init_carry_shape  # type: ignore[attr-defined]

    # Checks the `step` function.
    for step_input in step_fn.graph.input:
        step_input_type = step_input.type.tensor_type
        shape = tuple(dim.dim_value for dim in step_input_type.shape.dim)
        expected_shape = PyInputType(step_input.name).get_shape(metadata)
        if shape != tuple(expected_shape):
            raise ValueError(f"Expected shape {expected_shape} for input `{step_input.name}`, got {shape}")

    if len(step_fn.graph.output) != 2:
        raise ValueError(f"Step function must have exactly 2 outputs, got {len(step_fn.graph.output)}")

    output_actions = step_fn.graph.output[0]
    actions_shape = tuple(dim.dim_value for dim in output_actions.type.tensor_type.shape.dim)
    if actions_shape != (num_joints,):
        raise ValueError(f"Expected output shape {num_joints} for output `{output_actions.name}`, got {actions_shape}")

    output_carry = step_fn.graph.output[1]
    output_carry_shape = tuple(dim.dim_value for dim in output_carry.type.tensor_type.shape.dim)
    if output_carry_shape != init_carry_shape:
        raise ValueError(f"Expected carry shape {init_carry_shape} for output carry, got {output_carry_shape}")

    buffer = io.BytesIO()

    with tarfile.open(fileobj=buffer, mode="w:gz") as tar:

        def add_file_bytes(name: str, data: bytes) -> None:  # noqa: ANN401
            info = tarfile.TarInfo(name=name)
            info.size = len(data)
            tar.addfile(info, io.BytesIO(data))

        add_file_bytes("init_fn.onnx", init_fn.SerializeToString())
        add_file_bytes("step_fn.onnx", step_fn.SerializeToString())
        add_file_bytes("metadata.json", metadata.to_json().encode("utf-8"))

    buffer.seek(0)

    return buffer.read()



"""Defines the kinfer API."""

import os

if "ORT_DYLIB_PATH" not in os.environ:
    from pathlib import Path

    import onnxruntime as ort

    LIB_PATH = next((Path(ort.__file__).parent / "capi").glob("libonnxruntime.*"), None)
    if LIB_PATH is not None:
        os.environ["ORT_DYLIB_PATH"] = LIB_PATH.resolve().as_posix()

from .rust_bindings import get_version

__version__ = get_version()



# mypy: disable-error-code="import-untyped"
#!/usr/bin/env python
"""Setup script for the project."""

import re
import subprocess
from typing import List

from setuptools import find_packages, setup
from setuptools.command.build_ext import build_ext
from setuptools.command.build_py import build_py
from setuptools_rust import Binding, RustExtension

with open("README.md", "r", encoding="utf-8") as f:
    long_description: str = f.read()


with open("kinfer/requirements.txt", "r", encoding="utf-8") as f:
    requirements: List[str] = f.read().splitlines()


with open("Cargo.toml", "r", encoding="utf-8") as fh:
    version_re = re.search(r"^version = \"([^\"]*)\"", fh.read(), re.MULTILINE)
assert version_re is not None, "Could not find version in Cargo.toml"
version: str = version_re.group(1)

requirements_dev = [
    "black",
    "darglint",
    "mypy",
    "pytest",
    "ruff",
    "types-tensorflow",
]

requirements_pytorch = [
    "torch",
]

requirements_jax = [
    "tensorflow",
    "tf2onnx>=1.16.0",
    "jax",
    "equinox",
    "numpy<2",
]


class RustBuildExt(build_ext):
    def run(self) -> None:
        subprocess.run(["cargo", "run", "--bin", "stub_gen"], check=True)
        super().run()


class CustomBuild(build_py):
    def run(self) -> None:
        self.run_command("build_ext")
        super().run()


setup(
    name="kinfer",
    version=version,
    description="Tool to make it easier to run a model on a real robot",
    author="K-Scale Labs",
    url="https://github.com/kscalelabs/kinfer.git",
    rust_extensions=[
        RustExtension(
            target="kinfer.rust_bindings",
            path="kinfer/rust_bindings/Cargo.toml",
            binding=Binding.PyO3,
        ),
    ],
    setup_requires=["setuptools-rust"],
    zip_safe=False,
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires=">=3.11",
    install_requires=requirements,
    extras_require={
        "dev": requirements_dev,
        "pytorch": requirements_pytorch,
        "jax": requirements_jax,
        "all": requirements_dev + requirements_pytorch + requirements_jax,
    },
    include_package_data=True,
    packages=find_packages(),
    cmdclass={
        "build_ext": RustBuildExt,
        "build_py": CustomBuild,
    },
)
